#ifndef INPUT_H
#define INPUT_H

#include "stdbool.h"

//! Input type, second nibble is an index.
//! First nibble is a bitmask of what type of listener will work.
//! Where 0b0001 is float and 0b0010 is bool listeners.
typedef enum InputType {
    INPUT_LISTENER_KEY            = 0x11,
    INPUT_LISTENER_GAMEPAD_BUTTON = 0x21,
    INPUT_LISTENER_GAMEPAD_AXIS   = 0x32,
    INPUT_LISTENER_MOUSE          = 0x42,
} InputType;

typedef void(*BoolListener)(void*,bool);
typedef void(*FloatListener)(void*,float);

typedef struct InputListener {
    void *object;
    BoolListener button_listener;
    FloatListener axis_listener;
} InputListener;

#define ButtonInputListener(object_, function_) ((InputListener){.object = object_, .button_listener = (void(*)(void*, bool))function_, .axis_listener = NULL})
#define AxisInputListener(object_, function_)   ((InputListener){.object = object_, .button_listener = NULL, .axis_listener = (void(*)(void*, float))function_})

extern void InitializeInputSubsystem();
extern void CleanInputSubsystem();

extern void Internal_UpdateInput();

extern void AddAction(char const *action_name, InputType dev_type, int device, int input);

extern void AddListener(char const *action, InputListener listener);
extern void RemoveListener(char const *action, void *object);
extern void RemoveAllListeners(void *object);

#endif // !INPUT_H
